import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;
import javax.swing.table.*;

public class Mission extends java.lang.Object
{
    private Vector<NavPoint> navpoints;
    private int currentnavpoint;
    private String sector;
    private String briefing;
    private boolean friendly_eject_pickup;
    private String name;
    private Vector objectives;

    public void setCurrentNav(float x, float z, float scale)
    {
        int i;
        NavPoint currentnav;
        float delta = 40.0f * scale;
       
        
        for(i=0; i < navpoints.size(); i++)
        {
        	currentnav =  navpoints.elementAt(i);
            if(Math.abs(currentnav.getX() - x) < delta
             && Math.abs(currentnav.getZ() - z) < delta)
            {
            	currentnavpoint = i;
                return;

            }
        }    	    	
    }
    
    public NavPoint getNavAt(int i)
    {
    	return navpoints.elementAt(i);
    }
    
    public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getBriefing()
    {
      return briefing;
    }

    public void setBriefing(String newbriefing)
    {
      briefing=newbriefing;
      if(briefing == null)
      {
         briefing = "";
      }
    }

    public String getSector()
    {
       return sector;
    }
    public void setSector(String newsector)
    {
       sector = newsector;
    }

    public MissionValidStatus isValid()
    {
        NavPoint first =  navpoints.elementAt(0);
        NavPoint last =   navpoints.elementAt(getLastNavNumber() );
        int i;
        boolean playerfound=false;
        boolean landfound = false;
        Ship thisship;
        MissionValidStatus status = new MissionValidStatus();

        for(i=0; i < first.getNumShips() && (! playerfound); i++)
        {
            thisship = first.getShip(i);
            if(thisship.isPlayer() )
              playerfound = true;
        }
        for(i=0; i < last.getNumShips() && (! landfound); i++)
        {
            thisship = last.getShip(i);
            if(thisship.getLand() )
              landfound = true;
        }
        
        // check ships for valid scriptid
        for(NavPoint nav : navpoints)
        {
        	for(i=0; i < nav.getNumShips(); i++)
        	{
        		Ship ship = nav.getShip(i);
        		
        		if(! ship.isScriptValid())
        		{
        			status.valid = false;
        			status.reason = "Ship "+ ship.getType()+ " pilot=" + ship.getName() + " comm=" + ship.getComm() +

        			" did not have a script id, but needed one";
        			return status;
        		}
        	}
        }
        

        status.valid = landfound && playerfound;
        if(!landfound)
        {
        	status.reason = "No landing ship found";
        }
        else if(! playerfound)
        {
        	status.reason = "No player ship found";
        }
        
        return status;
    }

    public int getLastNavNumber()
    {
        return navpoints.size() - 1;
    }

    public Mission()
    {
      friendly_eject_pickup = false;       
      navpoints = new Vector<NavPoint>();
      objectives = new Vector();
      name = "";
      addNavPoint();
    }
    public NavPoint getCurrentNav()
    {
      return (NavPoint) navpoints.elementAt(currentnavpoint);
    }
    public void addNavPoint()
    {
        navpoints.addElement(new NavPoint() );
        currentnavpoint = navpoints.size() - 1;
    }

    public void removeCurrentNav()
    {
        if(navpoints.size() == 1)
        {
          return ;
        }
        else
        {
          navpoints.removeElementAt(currentnavpoint);
          currentnavpoint = navpoints.size() - 1;
        }
    }
    public void save(String filename) throws IOException
    {
                   FileWriter filewriter = new FileWriter(filename);
                   PrintWriter pw = new PrintWriter(filewriter);
                   pw.write(  getSourceString() );
                   pw.close();


    }
    public void load(String filename) throws Exception, Throwable
    {
        friendly_eject_pickup = false;       
        MissionXmlAdapter adapter = new MissionXmlAdapter(filename,this);
        navpoints = adapter.getNavPoints();
        currentnavpoint = 0;
    }

    public void clear()
    {
      navpoints = new Vector<NavPoint>();
      addNavPoint();
    }
    public void nextNav()
    {
     currentnavpoint++;
     if(currentnavpoint >= navpoints.size() )
     {
      currentnavpoint = 0;
     }
    }
    public void prevNav()
    {
     currentnavpoint--;
     if(currentnavpoint < 0 )
     {
      currentnavpoint = navpoints.size() - 1;
     }

    }
    public int getCurrentNavNumber()
    {
     return currentnavpoint;
    }
    private String getSourceString()
    {
        int navnum;
        StringBuffer buffer = new StringBuffer();
        buffer.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
        buffer.append("<mission name = \"" + name + "\" sector=\"" + sector + "\" " +
                  "can_eject=\"" + friendly_eject_pickup + "\" " + 
              "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"mission.xsd\""
                 +
        		">\n" );

        int numnavpoints = navpoints.size();
        buffer.append("  <navpoints number=\"" + numnavpoints+ "\">\n" );
        for(navnum = 0; navnum < numnavpoints; navnum++)
        {
            NavPoint currentnav = (NavPoint) navpoints.elementAt(navnum);
            int numships = currentnav.getNumShips();
            int numwaves = currentnav.getNumWaves();

            buffer.append("    <navpoint numships=\"" +  numships + "\" numwaves=\"" + numwaves + "\""); 
            if(currentnav.getName().length() > 0)
            {            	
            	buffer.append(" name=\"" + currentnav.getName() + "\" ");
            }            		
            buffer.append(">\n");
            int i;
            for(i=0; i < numships; i++)
            {
               Ship currentship = currentnav.getShip(i);
               buffer.append("      <ship name=\"" +  currentship.getType() + "\" ");
               if(currentship.isPlayer() )
               {
                buffer.append("pilot=\"player\" ");
               }
               else if(currentship.getName() != null)
               {
                buffer.append("pilot=\""+currentship.getName()+"\" ");
               }
               if(currentship.getComm() != null)
               {
                 buffer.append("comm=\"" + currentship.getComm() + "\" ");
               }
               //////// video no longer necessary
               /*if(currentship.getVideo() != null)
               {
                  buffer.append("video=\"" + currentship.getVideo() + "\" ");
               }
               */
               
               if(! currentship.getAutopilot() )
               {
                 buffer.append("autopilot=\"false\" ");
               }
               else
               {
                  buffer.append("autopilot=\"true\" ");                  
               }
               if(currentship.getLand() )
               {
                 buffer.append("land=\"true\" ");
               }
               if(currentship.getMustSurvive() )
               {
                 buffer.append("must_survive=\"true\" ");
               }
               buffer.append( "alignment=\"" + currentship.getAlignment()+ "\" " );
               if(currentship.getAutopilot() && currentship.getLastAutopilotNav() != -1)
               {
                  buffer.append("last_autopilot_nav=\"" + currentship.getLastAutopilotNav()+ "\" " );                  
               }
               buffer.append(" wave=\""+ currentship.getWave() + "\"");
               if(currentship.isRefueler())
               {
            	   buffer.append(" refueler=\"true\"");            	   
               }
               if(currentship.isJumpingIn())
               {
            	   buffer.append(" jumpin=\"true\"");            	   
               }
               
               if(currentship.getVariablename().length() > 0)
               {
            	   buffer.append(" scriptid=\"" + currentship.getVariablename() + "\"");            	   
               }
               
               buffer.append( ">\n" );
               buffer.append("        <location>\n");

               buffer.append("          <point  x=\"" + currentship.getX() + "\" " +
                                                "y =\"" + currentship.getY() + "\" " +
                                                "z =\"" + currentship.getZ() + "\" " +
                                         "/>\n");
               buffer.append("        </location>\n");
               buffer.append("        <rotation>\n");

               buffer.append("          <point  x=\"" + currentship.getRx() + "\" " +
                                                "y =\"" + currentship.getRy() + "\" " +
                                                "z =\"" + currentship.getRz() + "\" " +
                                         "/>\n");
               buffer.append("        </rotation>\n");
               
               if(currentship.getScript().isValid())
               {      
            	   Script shipscirpt = currentship.getScript();
            	   buffer.append("        <script ");
            	   if(shipscirpt.getEntryname().length() > 0)
            	   {
            		   buffer.append("oncreate=\"" + shipscirpt.getEntryname() + "\" ");
            	   }
            	   if(shipscirpt.getExitname().length() > 0)
            	   {
            		   buffer.append("ondestroy=\"" + shipscirpt.getExitname() + "\" ");
            	   }
            	   if(shipscirpt.getDuringname().length() > 0)
            	   {
            		   buffer.append("during=\"" + shipscirpt.getDuringname() + "\" ");
            	   }            	   
            	   buffer.append(" />\n");            	   
               }
               buffer.append("      </ship>\n");
            }
            if(currentnav.isHasAsteroids())
            {
             buffer.append("      <asteroids radius=\"0.0\" number=\"1000\">\n        <point x=\"0.0\" y=\"0.0\" z=\"0.0\"/>\n      </asteroids>\n");                 
            }
            if(currentnav.getBeforeComms().size() != 0)
            {
               buffer.append("      <onenter>\n");
               writeComms(buffer, currentnav.getBeforeComms());
               buffer.append("      </onenter>\n");               
            }            
            if(currentnav.getAfterComms().size() != 0)
            {
               buffer.append("      <onexit>\n");
               writeComms(buffer, currentnav.getAfterComms());               
               buffer.append("      </onexit>\n");               
               
            }
            if(currentnav.getScript().isValid())
            {      
         	   Script shipscirpt = currentnav.getScript();
         	   buffer.append("      <script ");
         	   if(shipscirpt.getEntryname().length() > 0)
         	   {
         		   buffer.append("oncreate=\"" + shipscirpt.getEntryname() + "\" ");
         	   }
         	   if(shipscirpt.getExitname().length() > 0)
         	   {
         		   buffer.append("ondestroy=\"" + shipscirpt.getExitname() + "\" ");
         	   }
         	   if(shipscirpt.getDuringname().length() > 0)
         	   {
         		   buffer.append("during=\"" + shipscirpt.getDuringname() + "\" ");
         	   }            	   
         	   buffer.append(" />\n");            	   
            }           
            buffer.append("      <location>\n");

            buffer.append("        <point  x=\"" + currentnav.getX() + "\" " +
                                           "y =\"" + currentnav.getY() + "\" " +
                                           "z =\"" + currentnav.getZ() + "\" " +
                                    "/>\n");
            buffer.append("      </location>\n");
            
            buffer.append("    </navpoint>\n");
        }
        buffer.append("  </navpoints>\n");
        if(objectives.size() > 0)
        {
        	buffer.append("  <objectives>\n");
        	int row;
        	for(row = 0; row < objectives.size(); row++)
        	{
        		Vector thisrow = (Vector) objectives.elementAt(row);
        		String name = (String) thisrow.elementAt(0);
        		String categrory = (String) thisrow.elementAt(1);
        		String thedefault = (String) thisrow.elementAt(2);
        		buffer.append("    <objective name=\"" + name + "\" category=\"" 
        				+ categrory + "\" default=\"" + thedefault + "\" />\n");
        	}
        	buffer.append("  </objectives>\n");        	        	
        }        
        buffer.append("</mission>\n");

        return buffer.toString();

    }

    private void writeComms(StringBuffer buffer, Vector comms)
    {
       int i;
       CommInfo comm;
       
       buffer.append("         <comms>\n");
       for(i=0; i < comms.size(); i++)
       {
          comm = (CommInfo) comms.elementAt(i);
          String alignment;
          if(comm.getFriendly())
          {
             alignment = "friend";
          }
          else
          {
             alignment = "enemy";
          }
          
          buffer.append("            <comm sound=\"" + comm.getSound() +
                "\" movie=\"" + comm.getVideo() +
                "\" label=\"" + comm.getName() +  "\" alignment=\"" + 
                alignment + 
                "\" />\n");
       }       
       buffer.append("         </comms>\n");       
    }
    
   /**
    * @param friendly_eject_pickup The friendly_eject_pickup to set.
    */
   public void setFriendlyEjectPickup(boolean friendly_eject_pickup)
   {
      this.friendly_eject_pickup = friendly_eject_pickup;
   }

   /**
    * @return Returns the friendly_eject_pickup.
    */
   public boolean getFriendlyEjectPickup()
   {
      return friendly_eject_pickup;
   }

public Vector getObjectives() {
	return objectives;
}

public void setObjectives(Vector objectives) {
	this.objectives = objectives;
}

}