
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.Label;
import java.util.Formatter;

enum SelectMode
{
	ADD,
	SELECT,
	PASTE	
}

public class MissionCanvas extends java.awt.Canvas implements
      java.awt.event.MouseListener, DragGestureListener, DragSourceListener,
      DropTargetListener, MouseMotionListener
{
   // 0 = add, 1 = select, 2 = paste
   protected SelectMode selectmode;
   protected Mission mission;
   protected MissionEditorFrame frame;
   protected float zoomfactor;
   private DragSource mysource;
   private Label statusbar;
   protected float fudgefactor;

   public void zoomIn()
   {
	  if(zoomfactor > .25f)
	  {		  
		  zoomfactor /= 2.0f;
	  }
   }

   public void zoomOut()
   {
	  if(zoomfactor < 128.0f)
	  {
		  zoomfactor *= 2.0f;
	  }

   }

   public MissionCanvas(Mission newmission, MissionEditorFrame newframe)
   {
      super();
      selectmode = SelectMode.SELECT;
      mission = newmission;
      addMouseListener(this);
      frame = newframe;
      zoomfactor = 2.0f;

      mysource = new DragSource();

      new DropTarget(this, DnDConstants.ACTION_COPY_OR_MOVE, this);

      mysource.createDefaultDragGestureRecognizer(this,
            DnDConstants.ACTION_COPY_OR_MOVE, this);
      addMouseMotionListener(this);
      setBackground(Color.BLACK);
      fudgefactor = 100.0f; 

   }

   public void mouseClicked(MouseEvent e)
   {
   }

   public void mouseEntered(MouseEvent e)
   {

   }

   public void mouseExited(MouseEvent e)
   {

   }

   public void paint(Graphics g)
   {
      NavPoint nav = mission.getCurrentNav();
      int i;
      int numships = nav.getNumShips();
      Ship ship;

      Dimension mysize = getSize();
      g.setColor(Color.black);
      g.fillRect(0, 0, mysize.width, mysize.height);
      
      
      Color darkgreen = new Color(0.0f, .5f, 0.0f);
	   Color fcgreen = new Color(0.1f, .86f, 0.4f);
	   if(zoomfactor < 16.0)
	   {
		   renderGrid(g, 20.0f, darkgreen);
	   }
      
      renderGrid(g, 200.0f, fcgreen);
      
      // center
      renderGrid(g, 200000.0f, Color.YELLOW);
      
      for (i = 0; i < numships; i++)
      {
         ship = nav.getShip(i);
         paintShip(g, ship.isFriend(), ship.getX(), ship
               .getZ(), nav.getCurrentShip() == ship,
               ship.isNeutral(), ship.getRy());
      }
   }
   
   protected void renderGrid(Graphics g, float gamewidth, Color color)
   {
	   if(getScale() == 0.0)
	   {
		   return;		   
	   }
	   
       //render the grid lines
	   double size =  gamewidth / getScale();	   
	   
	   Dimension mysize = getSize();
	   
	   int centerx = mysize.width / 2;
	   int centery = mysize.height / 2;
	   
	   float position = centerx;

	  
	   g.setColor(color);
	   
	   while(position < mysize.width)
	   {
		   g.drawLine((int) position, 0, (int) position, mysize.height);
		   position += size;
	   }
	   position = centerx;
	   while(position > 0)
	   {
		   g.drawLine((int)position, 0, (int)position, mysize.height);
		   position -= size;
	   }	   
	   
	   position = centery;
	   while(position < mysize.height)
	   {
		   g.drawLine(0, (int)position, mysize.width, (int)position);
		   position += size;
	   }
	   position = centery;
	   while(position > 0)
	   {
		   g.drawLine(0, (int)position, mysize.width, (int)position);
		   position -= size;
	   }
	   
	   
   }
   
   private void paintShip(Graphics g, boolean friendly, float x, float y,
         boolean selectedship,
         boolean neutralship,
         float ry)
   {
      int intx, inty;
      int centerx, centery;
      int radius;

      radius = 8;
      if (selectedship)
         radius = 14;

      Dimension mysize = getSize();
      centerx = mysize.width / 2;
      centery = mysize.height / 2;

      float min;

      if (mysize.width < mysize.height)
         min = mysize.width;
      else
         min = mysize.height;

      float scale = zoomfactor * fudgefactor / min;

      intx = centerx + (int) (x * 1 / scale);
      inty = centery + (int) (y * 1 / scale);
      if (friendly)
      {
         g.setColor(Color.blue);
      }
      else if(neutralship)
      {
         g.setColor(Color.gray);     
      }
      else
      {
         g.setColor(Color.red);
      }
      g.fillOval(intx, inty, radius, radius);
      double theta = Math.toRadians(ry);
      double yaddition = Math.cos(theta) * radius ;
      double xaddition = Math.sin(theta) * radius ;
      g.drawLine(intx + radius /2, inty + radius / 2, intx + radius /2 +(int) xaddition,
    		  inty + (int)yaddition + radius / 2);
   }

   public void mouseReleased(MouseEvent e)
   {

   }

   public void mousePressed(MouseEvent e)
   {
      int centerx, centery;

      Dimension mysize = getSize();
      centerx = mysize.width / 2;
      centery = mysize.height / 2;

      float min;

      if (mysize.width < mysize.height)
         min = mysize.width;
      else
         min = mysize.height;

      float scale = zoomfactor * fudgefactor / min;

      if (selectmode == SelectMode.SELECT)
      {
         mission.getCurrentNav().setCurrentShip(
               ((float) (e.getX() - centerx)) * scale,
               ((float) (e.getY() - centery)) * scale, scale);

      }
      else if(selectmode == SelectMode.PASTE)
      {
         Ship pasteship = frame.getPasteShip();
         if(pasteship != null)
         {
            mission.getCurrentNav().addShip(
                  ((float) (e.getX() - centerx)) * scale,
                  ((float) (e.getY() - centery)) * scale,
                  pasteship);
            
         }
      }
      else
      {
         mission.getCurrentNav().addShip(
               ((float) (e.getX() - centerx)) * scale,
               ((float) (e.getY() - centery)) * scale,
               frame.getCurrentShipChoice());
         frame.onShipAdded();
      }
      selectmode = SelectMode.SELECT;
      setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      paint(getGraphics());
      frame.onShipSelectionChange();

   }

   public void setCreateMode()
   {
      selectmode = SelectMode.ADD;
      setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
   }

   public void setPasteMode()
   {
      selectmode = SelectMode.PASTE;
      setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));      
   }
   
   /*
    * (non-Javadoc)
    * 
    * @see java.awt.dnd.DragGestureListener#dragGestureRecognized(java.awt.dnd.DragGestureEvent)
    */
   public void dragGestureRecognized(DragGestureEvent e)
   {
      Ship ship;

      ship = mission.getCurrentNav().getCurrentShip();

      if (ship != null)
      {
         e.startDrag(DragSource.DefaultCopyDrop, new ShipTranferable(), this);

      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see java.awt.dnd.DragSourceListener#dragEnter(java.awt.dnd.DragSourceDragEvent)
    */
   public void dragEnter(DragSourceDragEvent arg0)
   {
      // TODO Auto-generated method stub

   }

   /*
    * (non-Javadoc)
    * 
    * @see java.awt.dnd.DragSourceListener#dragOver(java.awt.dnd.DragSourceDragEvent)
    */
   public void dragOver(DragSourceDragEvent arg0)
   {
      // TODO Auto-generated method stub

   }

   /*
    * (non-Javadoc)
    * 
    * @see java.awt.dnd.DragSourceListener#dropActionChanged(java.awt.dnd.DragSourceDragEvent)
    */
   public void dropActionChanged(DragSourceDragEvent arg0)
   {
      // TODO Auto-generated method stub

   }

   /*
    * (non-Javadoc)
    * 
    * @see java.awt.dnd.DragSourceListener#dragExit(java.awt.dnd.DragSourceEvent)
    */
   public void dragExit(DragSourceEvent arg0)
   {
      // TODO Auto-generated method stub

   }

   /*
    * (non-Javadoc)
    * 
    * @see java.awt.dnd.DragSourceListener#dragDropEnd(java.awt.dnd.DragSourceDropEvent)
    */
   public void dragDropEnd(DragSourceDropEvent arg0)
   {
      // TODO Auto-generated method stub

   }

   /*
    * (non-Javadoc)
    * 
    * @see java.awt.dnd.DropTargetListener#dragEnter(java.awt.dnd.DropTargetDragEvent)
    */
   public void dragEnter(DropTargetDragEvent arg0)
   {
      // TODO Auto-generated method stub

   }

   /*
    * (non-Javadoc)
    * 
    * @see java.awt.dnd.DropTargetListener#dragOver(java.awt.dnd.DropTargetDragEvent)
    */
   public void dragOver(DropTargetDragEvent arg0)
   {
      // TODO Auto-generated method stub

   }

   /*
    * (non-Javadoc)
    * 
    * @see java.awt.dnd.DropTargetListener#dropActionChanged(java.awt.dnd.DropTargetDragEvent)
    */
   public void dropActionChanged(DropTargetDragEvent arg0)
   {
      // TODO Auto-generated method stub

   }

   /*
    * (non-Javadoc)
    * 
    * @see java.awt.dnd.DropTargetListener#dragExit(java.awt.dnd.DropTargetEvent)
    */
   public void dragExit(DropTargetEvent arg0)
   {
      // TODO Auto-generated method stub

   }

   /*
    * (non-Javadoc)
    * 
    * @see java.awt.dnd.DropTargetListener#drop(java.awt.dnd.DropTargetDropEvent)
    */
   public void drop(DropTargetDropEvent dropevent)
   {
      dropevent.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
      Point location = dropevent.getLocation();

      handleDrop(location);
      dropevent.dropComplete(true);
      selectmode = SelectMode.SELECT;
      setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      paint(getGraphics());
      frame.onShipSelectionChange();

   }
   
   protected void handleDrop(Point location)
   {
	      int centerx, centery;
	      int min;
	   
	      Dimension mysize = getSize();
	      centerx = mysize.width / 2;
	      centery = mysize.height / 2;
	      if (mysize.width < mysize.height)
	         min = mysize.width;
	      else
	         min = mysize.height;

	      float scale = zoomfactor * fudgefactor / min;
	      Ship ship = mission.getCurrentNav().getCurrentShip();

	      ship.setX(((float) (location.getX() - centerx)) * scale);
	      ship.setZ(((float) (location.getY() - centery)) * scale);	   	   
   }

public void setStatusbar(Label statusbar) {
	this.statusbar = statusbar;
}

public void mouseDragged(MouseEvent arg0) {
	// TODO Auto-generated method stub
	
}

protected float getScale()
{
	int min;
	
    Dimension mysize = getSize();
    if (mysize.width < mysize.height)
       min = mysize.width;
    else
       min = mysize.height;

    float scale = zoomfactor * fudgefactor / min;
    return scale;
	
}

public void mouseMoved(MouseEvent arg0) {
	// TODO Auto-generated method stub
    int centerx, centery;
    int min;

    Dimension mysize = getSize();
    centerx = mysize.width / 2;
    centery = mysize.height / 2;
    if (mysize.width < mysize.height)
       min = mysize.width;
    else
       min = mysize.height;

    float scale = zoomfactor * fudgefactor / min;
    
	if (statusbar != null)
	{
		float newx = ((float) (arg0.getX() - centerx)) * scale;
		float newz = ((float) (arg0.getY() - centery)) * scale;
		String text = String.format("x=%.1f   z=%.1f", newx, newz);
		statusbar.setText(text);
	}
}

protected void setFudgefactor(float fudgefactor) {
	this.fudgefactor = fudgefactor;
}
}