
import java.io.*;
import org.xml.sax.*;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import java.util.*;

public class MissionXmlAdapter implements DocumentHandler, ErrorHandler
{
   private Ship ship;
   private Vector<NavPoint> navpoints;
   private Vector objectives;
   private NavPoint nav;
   private Mission mission;
   private Vector<CommInfo> comms;
   private float x, y, z;
   private Script script;
   boolean inship = false;

   public MissionXmlAdapter(String filename, Mission newmission)
         throws SAXParseException, NumberFormatException, SAXException,
         Throwable
   {
	  objectives = new Vector();
      Parser parser;
      SAXParserFactory spf = SAXParserFactory.newInstance();
      spf.setValidating(false);
      SAXParser sp = spf.newSAXParser();
      parser = sp.getParser();
      parser.setDocumentHandler(this);
      parser.setErrorHandler(this);
      String uri = "file:" + new File(filename).getAbsolutePath();
      navpoints = new Vector<NavPoint>();
      mission = newmission;
      parser.parse(uri);
   }

   public Vector<NavPoint> getNavPoints()
   {
      return navpoints;
   }

   public void error(SAXParseException e) throws SAXParseException
   {
      throw e;
   }

   public void fatalError(SAXParseException e) throws SAXParseException
   {
      throw e;
   }

   // dump warnings too
   public void warning(SAXParseException err) throws SAXParseException
   {
      /*
       * System.out.println ("** Warning" + ", line " + err.getLineNumber () + ",
       * uri " + err.getSystemId ()); System.out.println(" " + err.getMessage
       * ());
       */
   }

   public void setDocumentLocator(Locator l)
   {
      // we'd record this if we needed to resolve relative URIs
      // in content or attributes, or wanted to give diagnostics.
   }

   public void startDocument() throws SAXException
   {

   }

   public void endDocument() throws SAXException
   {

   }

   public void startElement(String tag, AttributeList attrs)
         throws SAXException, NumberFormatException
   {
      String name;
      String value;

      if (tag.equals("mission"))
      {
         for (int i = 0; i < attrs.getLength(); i++)
         {
            name = attrs.getName(i);
            value = attrs.getValue(i);

            if (name.equals("sector"))
            {
               mission.setSector(value);
            }
            else if(name.equals("can_eject") )
            {
               	mission.setFriendlyEjectPickup(value.equals("true") );
            }
            else if(name.equals("name"))
            {
            	mission.setName(value);
            }
         }
      }
      else if (tag.equals("ship"))
      {
    	 inship = true;
         ship = new Ship();
         for (int i = 0; i < attrs.getLength(); i++)
         {
            name = attrs.getName(i);
            value = attrs.getValue(i);

            if (name.equals("pilot"))
            {
               if (value.equals("player"))
               {
                  ship.setPlayer(true);
               }
               else
               {
                  ship.setPlayer(false);
                  ship.setName(value);
               }
            }
            else if (name.equals("name"))
            {
               ship.setType(value);
            }
            else if(name.equals("scriptid"))
            {
               ship.setVariablename(value);
            }
            else if (name.equals("alignment"))
            {
               ship.setAlignment(value);
            }
            else if (name.equals("comm"))
            {
               ship.setComm(value);
            }
            else if (name.equals("video"))
            {
               ship.setVideo(value);
            }            
            else if (name.equals("autopilot"))
            {
               ship.setAutopilot(value.equals("true"));
            }
            else if(name.equals("last_autopilot_nav"))
            {               
              int ivalue = Integer.valueOf(value).intValue(); 
              ship.setLastAutopilotNav(ivalue);  
            }
            else if(name.equals("wave"))
            {
              value = value.trim();
              ship.setWave( Integer.valueOf(value));
            }
            else if (name.equals("land"))
            {
               ship.setLand(value.equals("true"));
            }
            else if (name.equals("must_survive"))
            {
               ship.setMustSurvive(value.equals("true"));
            }
            else if(name.equals("refueler"))
            {
            	ship.setRefueler(value.equals("true"));
            }
            else if(name.equals("jumpin"))
            {
            	ship.setJumpingIn(value.equals("true"));            	
            }
         }
      }
      else if (tag.equals("navpoint"))
      {
         nav = new NavPoint();
         for (int i = 0; i < attrs.getLength(); i++)
         {
            name = attrs.getName(i);
            value = attrs.getValue(i);
            
            if(name.equals("numwaves"))
            {
            	nav.setNumWaves(Integer.valueOf(value));
            }
            else if(name.equals("name"))
            {
            	nav.setName(value);
            }
         }

      }
      else if (tag.equals("comms"))
      {
         comms = new Vector<CommInfo>();
      }      
      else if (tag.equals("asteroids"))
      {
         nav.setHasAsteroids(true);
      }
      else if(tag.equals("script"))
      {
    	  script = new Script();
          for (int i = 0; i < attrs.getLength(); i++)
          {
             name = attrs.getName(i);
             value = attrs.getValue(i);

             if (name.equals("oncreate"))
             {
            	script.setEntryname(value); 
             }
             else if(name.equals("ondestroy"))
             {
            	 script.setExitname(value);
             }
             else if(name.equals("during"))
             {
            	 script.setDuringname(value);
             }             
          }    	  
      }
      else if (tag.equals("point"))
      {
         for (int i = 0; i < attrs.getLength(); i++)
         {
            name = attrs.getName(i);
            value = attrs.getValue(i);

            if (name.equals("x"))
            {
               x = Float.valueOf(value).floatValue();
            }
            else if (name.equals("y"))
            {
               y = Float.valueOf(value).floatValue();
            }
            else if (name.equals("z"))
            {
               z = Float.valueOf(value).floatValue();
            }
         }
      }
      else if(tag.equals("objective"))
      {
    	  Vector temp = new Vector();
    	  String objname="";
    	  String objcategory="";
    	  String objdefault="";
          for (int i = 0; i < attrs.getLength(); i++)
          {
             name = attrs.getName(i);
             value = attrs.getValue(i);
             
             if(name.equals("name"))
             {
            	objname = value;             	 
             }
             else if(name.equals("category"))
             {
                objcategory = value;
             }
             else if(name.equals("default"))
             {
            	 objdefault = value;
             }             
          }   
          temp.add(objname);
          temp.add(objcategory);
          temp.add(objdefault);          
          objectives.add(temp);
      }

      else if (tag.equals("comm"))
      {
         CommInfo comminfo = new CommInfo();
         for (int i = 0; i < attrs.getLength(); i++)
         {
            name = attrs.getName(i);
            value = attrs.getValue(i);

            if (name.equals("sound"))
            {
               comminfo.setSound(value);
            }
            else if (name.equals("movie"))
            {
               comminfo.setVideo(value);
            }
            else if (name.equals("label"))
            {
               comminfo.setName(value);
            }
            else if (name.equals("alignment"))
            {
               comminfo.setFriendly(value.equals("friend"));
            }
         }
         comms.add(comminfo);
      }

   }

   public void endElement(String name) throws SAXException
   {
      if (name.equals("ship"))
      {    	  
         nav.addShip(ship);
         if(script != null)
         {
        	 ship.setScript(script);
         }
         
         script = null;
         inship = false;
      }
      else if(name.equals("location"))
      {
    	  if(inship)
    	  {
    		  ship.setX(x);
    		  ship.setY(y);
    		  ship.setZ(z);
    	  }
    	  else
    	  {
              nav.setX(x);
              nav.setY(y);
              nav.setZ(z);    	      		      		  
    	  }
      }
      else if(name.equals("rotation"))
      {
    	  ship.setRx(x);
    	  ship.setRy(y);
    	  ship.setRz(z);
      }
      else if (name.equals("navpoint"))
      {    	  
         navpoints.add(nav);
         if(script != null)
         {
        	 nav.setScript(script);
         }         
         script = null;
      }
      else if(name.equals("onenter"))
      {
         nav.setBeforeComms(comms);
      }
      else if(name.equals("onexit"))
      {
         nav.setAfterComms(comms);
      }
      else if(name.equals("objectives"))
      {
    	 mission.setObjectives(objectives);
      }
   }

   public void characters(char buf[], int offset, int len) throws SAXException
   {
      String newstring = new String(buf, offset, len);
      newstring = newstring.trim();
      if (newstring.length() != 0)
      {
         if (mission.getBriefing() == null)
         {

            mission.setBriefing(newstring);
         }
         else
         {
            mission.setBriefing(mission.getBriefing() + "\n" + newstring);
         }
      }
   }

   public void ignorableWhitespace(char buf[], int offset, int len)
         throws SAXException
   {

   }

   public void processingInstruction(String target, String data)
         throws SAXException
   {

   }

}

